/*
*  ALCATEL SpeedTouch USB modem block extract utility
*  Copyright (C) 2001 Benoit PAPILLAULT
*  
*  This program is free software; you can redistribute it and/or
*  modify it under the terms of the GNU General Public License
*  as published by the Free Software Foundation; either version 2
*  of the License, or (at your option) any later version.
*  
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*  Author   : Benoit PAPILLAULT <benoit.papillault@free.fr>
*  Creation : 25/04/2001
*  Refactor : Matthew Gream (matthew.gream@pobox.com)
*
*  Searching for the block is done using the best match for a start and end 
*  patterns. This has been tested with windows 1.3 drivers, linux 1.3 drivers
*  & linux 1.3.2 drivers (using binary available at the Alcatel website:
*  http://www.alcateldsl.com/ ).
*
* $Id: product-thomson.c,v 1.4 2003/11/23 22:26:15 matt Exp $
*/

#ifndef _PRODUCT_C_
#define _PRODUCT_C_

#include "pusb.h"
#include "product.h"
#include "report.h"
#include "extract.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/stat.h>
#include <errno.h>

/*****************************************************************************
* Local Prototypes
*****************************************************************************/

static unsigned char give_char(unsigned char c);

/* Alcatel ADSL SpeedTouch USB modem idVendor/idProduct */
#define ST_VENDOR  0x0572
#define ST_PRODUCT 0xCAFE

/* Endpoint numbers */
#define EP_INT      0x83
#define EP_DATA     0x02
#define EP_CODE     0x01

/* Timeout in milliseconds */
#define CTRL_TIMEOUT 2000
#define DATA_TIMEOUT 2000

#define OFFSET_7  0 /* size 1 */
#define OFFSET_b  1 /* size 8 */
#define OFFSET_d  9 /* size 4 */
#define OFFSET_e 13 /* size 1 */
#define OFFSET_f 14 /* size 1 */
#define TOTAL    15

#define SIZE_7 1
#define SIZE_b 8
#define SIZE_d 4
#define SIZE_e 1
#define SIZE_f 1

int dl_512_first = 1;
int sb = 0;
extern int cfg_signal_kernel;
int link_up = 0;

static const unsigned char data1[] = {

  0x88, 0xf8, 0x00, 0x00, 0x00, 0x00, 0xf2, 0x01, 
  0x0d, 0x00, 0x00, 0xea, 0x0c, 0x00, 0x00, 0xea, 
  0x0b, 0x00, 0x00, 0xea, 0x0a, 0x00, 0x00, 0xea, 
  0x09, 0x00, 0x00, 0xea, 0x08, 0x00, 0x00, 0xea, 
  0xa4, 0x00, 0x00, 0xea, 0x06, 0x00, 0x00, 0xea, 
  0x32, 0x11, 0xa0, 0xe3, 0x5c, 0x03, 0x9f, 0xe5, 
  0x00, 0x00, 0x81, 0xe5, 0x58, 0x13, 0x9f, 0xe5, 
  0x00, 0x00, 0xa0, 0xe3, 0x00, 0x00, 0x81, 0xe5, 
  0xf8, 0xff, 0xff, 0xea, 0x01, 0x00, 0xa0, 0xe3, 
  0x48, 0x13, 0x9f, 0xe5, 0x00, 0x00, 0x81, 0xe5, 
  0x3c, 0x13, 0x9f, 0xe5, 0x04, 0x00, 0xa0, 0xe3, 
  0x00, 0x00, 0x81, 0xe5, 0xd1, 0x00, 0xa0, 0xe3, 
  0x00, 0xf0, 0x29, 0xe1, 0x01, 0xd9, 0xa0, 0xe3, 
  0xd2, 0x00, 0xa0, 0xe3, 0x00, 0xf0, 0x29, 0xe1, 
  0x24, 0xd3, 0x9f, 0xe5, 0xd3, 0x00, 0xa0, 0xe3, 
  0x00, 0xf0, 0x29, 0xe1, 0x1c, 0xd3, 0x9f, 0xe5, 
  0x00, 0x00, 0xa0, 0xe3, 0x00, 0x10, 0xa0, 0xe3, 
  0xb0, 0x02, 0x9f, 0xe5, 0x08, 0x00, 0x80, 0xe2, 
  0xbc, 0x12, 0x9f, 0xe5, 0x00, 0x10, 0x80, 0xe5, 
  0x5f, 0x10, 0xe0, 0xe3, 0x5d, 0x30, 0xe0, 0xe3, 
  0x5c, 0x50, 0xe0, 0xe3, 0x5b, 0x70, 0xe0, 0xe3, 
  0xac, 0x02, 0x9f, 0xe5, 0xa4, 0x22, 0x9f, 0xe5, 
  0xa8, 0x42, 0x9f, 0xe5, 0xa8, 0x62, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x00, 0x30, 0x82, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x70, 0x86, 0xe5, 0x54, 0x02, 0x9f, 0xe5, 
  0x08, 0x00, 0x80, 0xe2, 0x5c, 0x12, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x6c, 0x02, 0x9f, 0xe5, 
  0x6c, 0x12, 0x9f, 0xe5, 0x00, 0x10, 0x80, 0xe5, 
  0x00, 0x10, 0xa0, 0xe3, 0x98, 0x02, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x94, 0x02, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x01, 0x09, 0xa0, 0xe3, 
  0x00, 0x10, 0xa0, 0xe3, 0x02, 0x26, 0xa0, 0xe3, 
  0x04, 0x30, 0xa0, 0xe3, 0x03, 0x10, 0x80, 0xe6, 
  0x02, 0x00, 0x50, 0xe1, 0xfc, 0xff, 0xff, 0x1a, 
  0x7f, 0x19, 0xa0, 0xe3, 0x1e, 0x30, 0xa0, 0xe3, 
  0x03, 0x3b, 0x83, 0xe2, 0x00, 0x00, 0xa0, 0xe3, 
  0x00, 0x2a, 0x83, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x14, 0x00, 0x50, 0xe3, 0x04, 0x20, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x02, 0xe0, 0xa0, 0xe3, 
  0x03, 0xeb, 0x8e, 0xe2, 0x0e, 0x20, 0xa0, 0xe1, 
  0x18, 0x00, 0x50, 0xe3, 0x04, 0x00, 0x00, 0xaa, 
  0x00, 0xca, 0x82, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x18, 0x00, 0x50, 0xe3, 0x04, 0xc0, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x40, 0x00, 0x50, 0xe3, 
  0x04, 0x00, 0x00, 0xaa, 0x00, 0x2a, 0x83, 0xe1, 
  0x01, 0x00, 0x80, 0xe2, 0x40, 0x00, 0x50, 0xe3, 
  0x04, 0x20, 0x81, 0xe4, 0xfa, 0xff, 0xff, 0xba, 
  0x01, 0x0a, 0x50, 0xe3, 0x04, 0x00, 0x00, 0xaa, 
  0x00, 0x2a, 0x8e, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x01, 0x0a, 0x50, 0xe3, 0x04, 0x20, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x7f, 0x09, 0xa0, 0xe3, 
  0x03, 0x40, 0xa0, 0xe3, 0x10, 0x0f, 0x02, 0xee, 
  0x10, 0x4f, 0x03, 0xee, 0x10, 0x0f, 0x07, 0xee, 
  0x10, 0x0f, 0x05, 0xee, 0x3d, 0x00, 0xa0, 0xe3, 
  0x10, 0x0f, 0x01, 0xee, 0x01, 0x73, 0xa0, 0xe3, 
  0x04, 0x00, 0xa0, 0xe3, 0x05, 0x00, 0xc7, 0xe5, 
  0x01, 0x00, 0xa0, 0xe3, 0x06, 0x00, 0xc7, 0xe5, 
  0x01, 0x00, 0xa0, 0xe3, 0x07, 0x00, 0xc7, 0xe5, 
  0x03, 0x00, 0xa0, 0xe3, 0x08, 0x00, 0xc7, 0xe5, 
  0x00, 0x00, 0x40, 0x01, 0x12, 0x88, 0xd4, 0xf2, 
  0x01, 0x00, 0x00, 0xce, 0x01, 0xa0, 0xe3, 0x09, 
  0x00, 0xc7, 0xe5, 0x80, 0x00, 0xa0, 0xe3, 0x01, 
  0x00, 0xc7, 0xe5, 0x53, 0x00, 0xa0, 0xe3, 0x00, 
  0xf0, 0x29, 0xe1, 0x94, 0x11, 0x9f, 0xe5, 0x21, 
  0x03, 0xa0, 0xe3, 0x00, 0x10, 0x80, 0xe5, 0x10, 
  0x10, 0xa0, 0xe3, 0x22, 0x01, 0xa0, 0xe3, 0x00, 
  0x10, 0x80, 0xe5, 0x04, 0x00, 0x8f, 0xe2, 0x01, 
  0x00, 0x80, 0xe2, 0x10, 0xff, 0x2f, 0xe1, 0x5e, 
  0x4f, 0xb8, 0x78, 0x40, 0x1e, 0xfc, 0xd1, 0x38, 
  0x78, 0xf9, 0x7a, 0x09, 0x02, 0xbc, 0x7a, 0x21, 
  0x43, 0x09, 0x02, 0x7c, 0x7a, 0x21, 0x43, 0x09, 
  0x02, 0x3c, 0x7a, 0x21, 0x43, 0x11, 0x38, 0x01, 
  0x28, 0x2d, 0xdb, 0x03, 0x28, 0x2b, 0xdc, 0x80, 
  0x00, 0x55, 0x4c, 0x24, 0x58, 0xa7, 0x46, 0x46, 
  0x4b, 0x0e, 0x33, 0x7a, 0x7b, 0x12, 0x02, 0x3c, 
  0x7b, 0x22, 0x43, 0x12, 0xd0, 0x14, 0x1c, 0xf4, 
  0x3c, 0xff, 0x3c, 0x0e, 0xd5, 0x40, 0x07, 0x02, 
  0xd0, 0x1c, 0x1c, 0x0b, 0x1c, 0x21, 0x1c, 0x52, 
  0x1e, 0x8c, 0x5c, 0x9c, 0x54, 0xfb, 0xd1, 0x3b, 
  0xa4, 0x14, 0x23, 0x23, 0x60, 0xc0, 0x0f, 0x64, 
  0x30, 0x00, 0xe0, 0x66, 0x20, 0x78, 0x70, 0xcb, 
  0xe7, 0x01, 0xa2, 0x10, 0x47, 0x00, 0x00, 0x01, 
  0xf0, 0xa0, 0xe1, 0x04, 0x20, 0x8f, 0xe2, 0x01, 
  0x20, 0x82, 0xe2, 0x12, 0xff, 0x2f, 0xe1, 0x60, 
  0x20, 0xf0, 0xe7, 0x04, 0xe0, 0x4e, 0xe2, 0x00, 
  0x40, 0x2d, 0xe9, 0x00, 0xe0, 0x4f, 0xe1, 0x07, 
  0x40, 0x2d, 0xe9, 0xa8, 0x10, 0x8f, 0xe2, 0x00, 
  0x00, 0x91, 0xe5, 0x01, 0x00, 0x20, 0xe2, 0x00, 
  0x00, 0x81, 0xe5, 0x00, 0x00, 0x50, 0xe3, 0x03, 
  0x00, 0x00, 0x0a, 0x0d, 0x10, 0xa0, 0xe3, 0xcc, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x82, 0xe5, 0x02, 
  0x00, 0x00, 0xea, 0x0f, 0x10, 0xa0, 0xe3, 0xbc, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x82, 0xe5, 0x01, 
  0x00, 0xa0, 0xe3, 0x90, 0x10, 0x9f, 0xe5, 0x00, 
  0x00, 0x81, 0xe5, 0x6c, 0x10, 0x8f, 0xe2, 0x00, 
  0x00, 0x91, 0xe5, 0x01, 0x00, 0x40, 0xe2, 0x00, 
  0x00, 0x81, 0xe5, 0x00, 0x00, 0x50, 0xe3, 0x02, 
  0x00, 0x00, 0x1a, 0x94, 0x00, 0x9f, 0xe5, 0x94, 
  0x10, 0x9f, 0xe5, 0x00, 0x00, 0x81, 0xe5, 0x90, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x92, 0xe5, 0x07, 
  0x40, 0xbd, 0xe8, 0x0e, 0xf0, 0x69, 0xe1, 0x00, 
  0x80, 0xfd, 0xe8, 0x00, 0x00, 0x00, 0x8c, 0x20, 
  0x00, 0x00, 0x88, 0x50, 0x00, 0x00, 0x01, 0x02, 
  0x00, 0x00, 0x00, 0xf4, 0x00, 0x00, 0x00, 0x39, 
  0x00, 0x00, 0x00, 0x10, 0x40, 0x08, 0x00, 0x00, 
  0x40, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x30, 
  0x4e, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x15, 
  0x80, 0x00, 0x00, 0x15, 0x2f, 0x8c, 0x9b, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x5d, 
  0x02, 0x00, 0x00, 0x5d, 0x02, 0x00, 0x00, 0x97, 
  0x02, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x40, 
  0x00, 0x40, 0x01, 0x64, 0x00, 0x50, 0x01, 0x70, 
  0x3e, 0x00, 0x00, 0xe0, 0x3c, 0x00, 0x00, 0x20, 
  0x00, 0x00, 0x90, 0x20, 0x00, 0x00, 0xa4, 0x85, 
  0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x78, 
  0x03, 0x00, 0x00, 0x58, 0x00, 0x50, 0x01, 0x70, 
  0xc3, 0x00, 0x00, 0x60, 0x00, 0x50, 0x01, 0x04, 
  0x00, 0x00, 0x84, 0x40, 0x01, 0x12, 0x08, 0x04, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x14

};

/******************************************************************************
*	Main Lib Function
******************************************************************************/

/*
* Function      : upload_block
* Return Values : -1 in case of error
* Description   : [...]
* NB            : This functions require the modem interface 2
*/
int upload_block(pusb_device_t fdusb,const char * filename)
{


	long uc_len;
	int n;
	unsigned char *uc_buf, buf[0x1000];
	/* buf should be at least 511 bytes */
	struct timeval start, stop;
	pusb_endpoint_t ep_code;
	struct extract_block_config config = {
		"???",
		{ 0x88, 0xf8, 0x00, 0x40, 0x00, 0x00, 0xf2, 0x01 }, 8,
		{ 0x04, 0x00, 0x40, 0x00, 0x00, 0x00, 0x01, 0x14 }, 8,
		0x0000,
		0x0000
	};

	if ((uc_buf = extract_block(&config,filename,&uc_len)) == NULL) {

		/*
		* this is a fatal error, either the file cannot be read
		* or the block is not inside
		*/
		report(0, REPORT_ERROR, "Unable to locate block\n");
		exit(-1);
		
	}

	/* open the EP_CODE endpoint */
	if ((ep_code = pusb_endpoint_open(fdusb,EP_CODE,O_RDWR)) == NULL) {
		report(0, REPORT_ERROR, "pusb_endpoint_open");
		exit(-1);
	}
	
	/* URB 7 */
	if(dl_512_first){

		memset(buf, 0, sizeof(buf));
		gettimeofday(&start,NULL);
		n = pusb_endpoint_read(ep_code,buf,0x200,DATA_TIMEOUT);
		gettimeofday(&stop,NULL);

		if (n < 0) {

			if (errno != ETIMEDOUT) {
				report(0, REPORT_ERROR, "BLOCK0\n");
				pusb_endpoint_close(ep_code);
				return(n);
			}

		}
		else {

			report(1, REPORT_INFO, "BLOCK0 : %6d bytes downloaded : OK\n", n);
			report(2, REPORT_DEBUG|REPORT_DUMP,"Downloaded in %f ms\n",
			       buf,
			       n,
			       (double)(stop.tv_sec -start.tv_sec) * 1000.0 +
			       (double)(stop.tv_usec - start.tv_usec) / 1000.0);

		}

	}

	/* URB 8 : both leds are static green */
	gettimeofday(&start,NULL);
	n = pusb_endpoint_write(ep_code,data1,sizeof(data1),DATA_TIMEOUT);
	gettimeofday(&stop,NULL);

	if (n < 0) {
		report(0, REPORT_ERROR, "BLOCK1\n");
		pusb_endpoint_close(ep_code);
		return(n);
	}
	else {
		report(1, REPORT_INFO, "BLOCK1 : %6d bytes   uploaded : OK\n", sizeof(data1));
		report(2, REPORT_DEBUG|REPORT_DUMP,"Downloaded in %f ms\n",
		       data1,
		       sizeof(data1),
		       (double)(stop.tv_sec -start.tv_sec) * 1000.0 +
		       (double)(stop.tv_usec - start.tv_usec) / 1000.0);

	}
	
	/*
	* After usb_bulk_write :
	*   + USB  led : blinking green
	*   + ADSL led : off
	*/
	
	/* URB 11 */
	memset(buf, 0, sizeof(buf));
	gettimeofday(&start,NULL);
	n = pusb_endpoint_read(ep_code,buf,0x200,DATA_TIMEOUT);
	gettimeofday(&stop,NULL);

	if (n < 0) {
		report(0, REPORT_ERROR, "BLOCK2\n");
		pusb_endpoint_close(ep_code);
		return(n);
	}
	else {

		report(1, REPORT_INFO, "BLOCK2 : %6d bytes downloaded : OK\n", n);
		report(2, REPORT_DEBUG|REPORT_DUMP,"Downloaded in %f ms\n",
		       buf,
		       n,
		       (double)(stop.tv_sec -start.tv_sec) * 1000.0 +
		       (double)(stop.tv_usec - start.tv_usec) / 1000.0);

	}
	
	
	/* URB 12 to 139 */
	gettimeofday(&start,NULL);
	n = pusb_endpoint_write(ep_code,uc_buf,uc_len,DATA_TIMEOUT);
	gettimeofday(&stop,NULL);
	
	if (n < 0) {
		report(0, REPORT_ERROR, "BLOCK3\n");
		pusb_endpoint_close(ep_code);
		return(n);
	}
	else {

		report(1, REPORT_INFO, "BLOCK3 : %6d bytes   uploaded : OK\n", n);
		report(2, REPORT_DEBUG, "Downloaded in %f ms\n", 
		       (double)(stop.tv_sec -start.tv_sec) * 1000.0 +
		       (double)(stop.tv_usec - start.tv_usec) / 1000.0);

	}

	/* Free the firmware allocated mem */
	free(uc_buf);

	/*
	* Before downloading block :
	*   + USB  led : blinking
	*   + ADSL led : off
	*   
	* During downloading:
	*   + USB  led : blinking green
	*   + ADSL led : off
	*   
	* After downloading block :
	*   + USB  led : static green
	*   + ADSL led : static red
	*/
	
	/* URB 142 */
	memset(buf, 0, sizeof(buf));
	gettimeofday(&start,NULL);
	n = pusb_endpoint_read(ep_code,buf,0x200,DATA_TIMEOUT);
	gettimeofday(&stop,NULL);
	
	if (n < 0) {
		report(0, REPORT_ERROR, "BLOCK4\n");
		pusb_endpoint_close(ep_code);
		return n;
	}
	else {

		report(1, REPORT_INFO, "BLOCK4 : %6d bytes downloaded : OK", n);
		report(2, REPORT_DEBUG|REPORT_DUMP, "Downloaded in %f ms.\n",
		       buf,
		       n,
		       (double)(stop.tv_sec -start.tv_sec) * 1000.0 +
		       (double)(stop.tv_usec - start.tv_usec) / 1000.0);

	}

	pusb_endpoint_close(ep_code);

	return(0);

}

/*
* Function      : get_reference
* Return Values : <0 on error, >0 on success
* Description   : Retrieves modem serial number
*/
int get_reference(pusb_device_t fdusb)
{

	unsigned char buf[0xe];
	int i,n;
	
	/*
	* URB 151 : this buffer match (if you reverse one byte with its neighbour)
	* the label on the manta box!!! in my case : 3EC18607CAAB06
	* 3EC 18607CAAB 09
	* NB : "My case " == "Benoit's case"
	*/
	
	/* original line */
	n = pusb_control_msg(fdusb,0xc0,0x26,0xf7,0,buf,sizeof(buf),CTRL_TIMEOUT);
	if (n<0)
		report(0, REPORT_ERROR, "get_reference: pusb_control_msg\n");
	else {
	
		char temp[128];

		for (i=0;i<n;i++) {
			int idx = ((i%2)==0) ? i+1 : i-1;
			temp[i] = give_char(buf[idx]);
		}

		temp[n] = '\0';

		report(1, REPORT_INFO, "Modem reference : %s\n", temp);

	}

	return(n);

}

/*
* Function      : modem_start_synchro
* Return Values : -1 on error, 0 on success
* Description   : This function initiates the modem synchronisation
*/
int modem_start_synchro(pusb_device_t fdusb)
{

	unsigned char buf[2];
	int n;

	n = pusb_control_msg(fdusb,0xc0,0x12,0x04,0x00,buf,sizeof(buf),CTRL_TIMEOUT);

	if (n < 0) {
		report(0, REPORT_ERROR, "modem_start_synchro: pusb_control_msg");
		return(-1);
	}

	report(2, REPORT_DEBUG|REPORT_DUMP, "modem_start_synchro :",
	       buf,
	       n);

	return(0);

}


/*
* Function      : swbuff ON/OFF
* Return Values : None
* Description   : Enables/disables software buffering
*                 Courtesy of Thomson Multimedia
*/  
void swbuff(pusb_device_t fdusb, int state)
{
   int n;

   n = pusb_control_msg(fdusb,0x40,0x32,state?0x01:0x00,0x00,NULL,0,100);

   report(0, n<0 ? REPORT_ERROR : REPORT_INFO, 
          state ? "Enabling SW buffering\n" : "Disabling SW buffering\n");
 }

/*
* Function      : get_state
* Return Values : -1 is returned on errors, 0 otherwise
* Description   : Retrieves modem state from the usb interrupt end point
* NB            : buf MUST BE at least 15 bytes.
*/
int get_state(pusb_device_t fdusb, unsigned char *buf)
{

	int ret;

	memset(buf,0,TOTAL);

	ret=pusb_control_msg(fdusb,0xc0,0x12,0x07,0x00,buf+OFFSET_7,SIZE_7,CTRL_TIMEOUT);
	if (ret<0) return(ret);

	ret=pusb_control_msg(fdusb,0xc0,0x12,0x0b,0x00,buf+OFFSET_b,SIZE_b,CTRL_TIMEOUT);
	if (ret<0) return(ret);

	ret=pusb_control_msg(fdusb,0xc0,0x12,0x0d,0x00,buf+OFFSET_d,SIZE_d,CTRL_TIMEOUT);
	if (ret<0) return(ret);

	ret=pusb_control_msg(fdusb,0xc0,0x01,0x0e,0x00,buf+OFFSET_e,SIZE_e,CTRL_TIMEOUT);
	if (ret<0) return(ret);

	ret=pusb_control_msg(fdusb,0xc0,0x01,0x0f,0x00,buf+OFFSET_f,SIZE_f,CTRL_TIMEOUT);
	if (ret<0) return(ret);

	return(0);

}

/*
* Function      : test_sequence
* Return Values : None
* Description   : This a magic sequence, totaly unknown to me [benoit],
*                 it comes from the Windows driver ... 
*/
void test_sequence(pusb_device_t fdusb)
{

	unsigned char buf[10];
	int n;

	/* URB 147 */
	buf[0] = 0x1c; buf[1] = 0x50;
	n = pusb_control_msg(fdusb,0x40,0x01,0x0b,0x00,buf,2,100);
	if (n<0)
		report(0, REPORT_ERROR, "URB147\n");

	/* URB 148 */
	buf[0] = 0x32; buf[1] = 0x00;
	n = pusb_control_msg(fdusb,0x40,0x01,0x02,0x00,buf,2,100);
	if (n<0)
		report(0, REPORT_ERROR,"URB148\n");

	/* URB 149 */
	buf[0] = 0x01; buf[1] = 0x00; buf[2] = 0x01;
	n = pusb_control_msg(fdusb,0x40,0x01,0x03,0x00,buf,3,100);
	if (n<0)
		report(0, REPORT_ERROR, "URB149\n");
	
	/* URB 150 */
	buf[0] = 0x01; buf[1] = 0x00; buf[2] = 0x01;
	n = pusb_control_msg(fdusb,0x40,0x01,0x04,0x00,buf,3,100);
	if (n<0)
		report(0, REPORT_ERROR, "URB150\n");

}

/*
* Function      : print_state
* Return Values : 0
* Description   : Print a readale modem state
* NB            : buf MUST be 15 bytes long
*/
int print_state(unsigned char *buf)
{
	time_t t = time(NULL);
	int down_speed = 0, up_speed = 0;
	
	report(2, REPORT_DEBUG,"Modem State\n",ctime(&t));
	report(2, REPORT_DEBUG|REPORT_DUMP,"c0 12 0007 : ", buf+OFFSET_7,SIZE_7);
	report(2, REPORT_DEBUG|REPORT_DUMP,"c0 12 000b : ", buf+OFFSET_b,SIZE_b);
	report(2, REPORT_DEBUG|REPORT_DUMP,"c0 12 000d : ", buf+OFFSET_d,SIZE_d);
	report(2, REPORT_DEBUG|REPORT_DUMP,"c0 01 000e : ", buf+OFFSET_e,SIZE_e);
	report(2, REPORT_DEBUG|REPORT_DUMP,"c0 01 000f : ", buf+OFFSET_f,SIZE_f);
	
	switch (buf[OFFSET_7]) {
	case 0x00:
		report(1, REPORT_INFO, "ADSL line is down\n");
		break;
	case 0x08:
		report(1, REPORT_INFO, "ADSL line is blocked?\n");
		break;
	case 0x10:
		report(1, REPORT_INFO, "ADSL line is synchronising\n");
		break;
	case 0x20:
		down_speed = buf[OFFSET_b] | (buf[OFFSET_b+1]<<8)
			| (buf[OFFSET_b+2]<<16) | (buf[OFFSET_b+3]<<24);
		up_speed = buf[OFFSET_b+4] | (buf[OFFSET_b+5]<<8)
			| (buf[OFFSET_b+6]<<16) | (buf[OFFSET_b+7]<<24);
		
	        if(	((down_speed & 0x0000ffff) == 0) &&
		((up_speed & 0x0000ffff) == 0)) {

		     down_speed>>=16;
		     up_speed>>=16;

	        } 

		report(0, REPORT_INFO, "ADSL line is up (%d kbit/s down | %d kbit/s up)\n",down_speed,up_speed);
		break;
	}
	
	return(0);

}

static unsigned char give_char(unsigned char c)
{

	if(c >= ' ' && c < 0x7f)
		return(c);
	else
		return('.');

}

/*
*         6 bytes are received when the line goes UP : a1 00 01 00 00 00
*         6 bytes are received for DOWN              : a1 00 00 00 00 00
*/
static int check_link_state(int len, const unsigned char* buf, int* state)
{
	if(len == 6) {

		/* The magic interrupt for "up state" */
		static const unsigned char up_int[6] = { 0xa1, 0x00, 0x01, 0x00, 0x00, 0x00};
		/* The magic interrupt for "down state" */
		static const unsigned char down_int[6] = { 0xa1, 0x00, 0x00, 0x00, 0x00, 0x00};

		if(!memcmp(up_int, buf, 6)) {
			*state = 1;
			return (0);
		}

		if(!memcmp(down_int, buf, 6)) {
			*state = 0;
			return (0);
		}
	
	}

	return (-1);
}

/*
* Function      : handle_endpoint_int
* Return Values : None
* Description   : This function handles the interrupt endpoint in a child
*                 process  and returns immediately.
*/
void handle_endpoint_int(pusb_device_t fdusb)
{

	pid_t child_pid;

	fflush(stdout);
	fflush(stderr);

	if ((child_pid=fork ()) == 0) {

		int ret, up, st;
		unsigned char lbuf[1024];
		pusb_endpoint_t ep_int;
		pid_t parent;

		parent = getppid();

		ep_int = pusb_endpoint_open(fdusb,EP_INT,O_RDONLY);
		
		if (ep_int == NULL) {
			perror("pusb_endpoint_open EP_INT");
			_exit (-1);
		}

#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) 
		/*
		 * BUG: unless we close the control endpoint, pppoa will
		 *      not be able to open the device.
		 */
		pusb_close(fdusb);
#endif

		up = 0;

		while(1) {

			ret = pusb_endpoint_read(ep_int,lbuf,sizeof(lbuf),0);
			
			if (ret < 0) {
				if(errno == ENODEV) {
					report(0, REPORT_INFO, "Device disconnected, shutting down");
					break;
				}
				else {
					report(0, REPORT_ERROR, "Error reading interrupts\n");
					continue;
				}
			}
			
			report(1, REPORT_INFO|REPORT_DUMP, "Received interrupt, len = %d\n", lbuf, ret, ret);

			if (!check_link_state(ret, lbuf, &st)) {

				if(st == 1) {
					/* We only signal the parent process the first time */
					if (!up) {
						report(1, REPORT_INFO, "Sending a signal to notify the adsl up state\n");
						kill(parent, SIGUSR2);
						up = 1;
					}
					if (cfg_signal_kernel)
						pusb_ioctl(fdusb, 1, 1, NULL);
				}

				if(st == 0 && cfg_signal_kernel) {
					pusb_ioctl(fdusb, 1, 2, NULL);
				}
	
			}

		}

		pusb_endpoint_close(ep_int);

		exit(-1);

	}

}
void signal_usr2(int signal)
{
	link_up = ~link_up;
}

#endif /* #ifndef _PRODUCT_C_ */
